* File of the Project "PROJECTNAMET" - 1_additional_vars.do
* Content: Generate additional biographical variables - Code is directly from Eberle, Schmucker, and Seth from IAB webpage



/*------------------------------------------------------------------------------------------------*\
  GENERATE ADDITIONAL BIOGRAPHIC VARIABLES FROM LONGITUDINAL DATA

	First day in employment (ein_erw):
		- Date of entry into first employment
		- Periods of vocational training are ignored (stib = 0)
		  --> Missing for persons continuously in vocational training in SIAB
		- Entry into first employment can be later than entry in first establishment or job, since these variables include periods of vocational training

	Number of days in employment (tage_erw):
		- Total number of days a person was employed until the end of the observation or until reference date 
		- Periods of vocational training are ignored (stib = 0)
		  --> Value of 0 for persons who are in vocational training throughout
		  
	First day in establishment (ein_bet):
		- Date of entry into establishment
		- Includes periods of vocational training
		- Not affected by interruptions of employment
		  --> unique for every combination of person and establishment

	Number of days in establishment (tage_bet):
		- Number of days in establishment until the end of the observation or until reference date
		- Includes periods of vocational training 
		- Gaps are subtracted

	First day in job (ein_job):
		- Start date of current job
		- Periods of vocational training are considered as separate jobs (even if there is no time lag between training and job)
		- Re-employment after interruptions in the same establishment are considered as new jobs if:
		  a) the reason of notification to social security agency implies end of employment: (grund = 30, 34, 40, 49) 
			 + Time gap > 92 days
		  b) an other reason of notification exists
			 + Time gap > 366 days

	 Number of days in job (tage_job):
		- Number of days in current job until the end of the observation or until reference date
		- see ein_job

	 Number of benefit receipts (anz_lst):
		- Number of benefit receipts accoring to SGB II or SGB III
		- Sources LeH LHG XLHG
		- Time gaps are ignored if interruption < 10 days
		- Change of benefit type doesn't count as new benefit receipt

	Number of days with benefit receipt (tage_lst):
		- Duration of benefit receipts until the end of the observation or until reference date
		- see anz_lst
		- Gaps are subtracted

\*------------------------------------------------------------------------------------------------*/

clear
*set max_memory 20g
cap log close
log using "$log/1_additional_vars.log", replace

* LOAD SIAB DATA
use "$orig/siab_7510_v1.dta", clear

bys persnr: gen sample=runiform() if _n==1
bys persnr: egen sample2=max(sample)
count
keep if sample2<.99
count
drop sample sample2


keep persnr betnr quelle stib erwstat begorig begepi endorig endepi tentgelt gebjahr erwstat spell grund beruf frau bild wo_bula


*---------------------------------------*
* OBSERVATION COUNTER                   *
*---------------------------------------*

* OBSERVATION COUNTER PER EPISODE AND SOURCE
bysort persnr begepi quelle (spell): gen byte level1 = _n-1

* OBSERVATION COUNTER PER EPISODE
bysort persnr begepi (spell): gen byte level2 = _n-1

*---------------------------------------*
* FIRST DAY IN EMPLOYMENT  (ein_erw)    *
*---------------------------------------*

sort persnr spell


* TAG EMPLOYMENTS (WITHOUT VOCATIONAL TRAINING)
gen emp = 1 if stib != 0 & quelle==1

* TAG FIRST EMPLOYMENT
bysort persnr emp (begorig): gen h = begorig[1] if !missing(emp)

* ASSIGN DATE OF FIRST EMPLOYMENT TO ALL OBSERVATIONS
bysort persnr: egen ein_erw = max(h)
replace ein_erw = .n if ein_erw ==.
format ein_erw %td
drop h

*-----------------------------------------*
* NUMBER OF DAYS IN EMPLOYMENT (tage_erw) *
*-----------------------------------------*

* AUXILIARY VARIABLE COUNTS EMPLOYMENT OBSERVATIONS PER EPISODE (WITHOUT VOCATIONAL TRAINING)
bysort persnr emp begepi (spell): gen nrE = _n if stib != 0 & quelle==1

* AUXILIARY VARIABLE FOR EMPLOYMENT DURATIONS (EXCLUDING PARALLEL EPISODES)
gen d = endepi - begepi +1 if emp == 1 & nrE==1

* RUNNING TOTAL OF JOB DURATIONS
bysort persnr (begepi nrE): gen tage_erw = sum(d)
sort persnr spell
drop d emp nrE

*--------------------------------------*
* FIRST DAY IN ESTABLISHMENT (ein_bet) *
*--------------------------------------*

bysort persnr betnr: egen ein_bet = min(begepi) if !missing(betnr)
format ein_bet %td

*--------------------------------------------*
* NUMBER OF DAYS IN ESTABLISHMENT (tage_bet) *
*--------------------------------------------*

* AUXILIARY VARIABLE MARKS DUPLICATE OBSERVATIONS PER ESTABLISHMENT AND EPISODE
bysort persnr betnr begepi endepi (spell): gen nr = _n if !missing(betnr)

* CALCULATION OF THE DURATION
gen dauer = endepi - begepi +1

* REPLACE DURATION OF DUPLICATE OBSERVATIONS AS 0
replace dauer = 0 if nr !=1

* RUNNING TOTAL OF DAYS IN ESTABLISHMENT
bysort persnr betnr (spell): gen tage_bet = sum(dauer) if !missing(betnr)
drop dauer

*----------------------------*
* FIRST DAY IN JOB (ein_job) *
*----------------------------*

sort persnr betnr spell

* GENERATE AUXILIARY VARIABLE IN ORDER TO COUNT PERIODS OF VOCATIONAL TRAINING AS SEPARATE JOBS
gen byte ausbild = (stib==0) if quelle ==1

* SORT OBSERVATIONS BY PERSON AND ESTABLISHMENT (VOCATIONAL TRAINING SEPARATE), MAIN EMPLOYMENT COMES LAST (TO COMPARE IT WITH NEXT OBSERVATION)
gsort persnr ausbild betnr begepi -level2

* MARK SUBSEQUENT EPISODES OF JOB (FIRST OBSERVATION MISSING)
gen job = 1 if persnr == persnr[_n-1] & betnr == betnr[_n-1] & ausbild == ausbild[_n-1] & !missing(betnr)

* COUNT AS NEW JOB IF EMPLOYER REPORTED END OF EMPLOYMENT AND GAP > 92 DAYS
* COUNT AS NEW JOB IF GAP > 366 DAYS
gen end = 1 if inlist(grund, 30, 34, 40, 49)
gen gap = begepi - endepi[_n-1] -1 if job ==1 
replace job = . if end[_n-1] ==1 & gap > 92
replace job = . if gap > 366

* GENERATE START DATE OF JOB, THEN COPY IT TO SUBSEQUENT OBSERVATIONS OF SAME JOB
gen ein_job = begepi if !missing(betnr)
replace ein_job = ein_job[_n-1] if job ==1
format ein_job %td
drop end gap

*----------------------------------*
* NUMBER OF DAYS IN JOB (tage_job) *
*----------------------------------*

sort persnr betnr spell

* AUXILIARY VARIABLE FOR NUMBER OF PARALLEL EPISODES PER JOB (VOCATIONAL TRAINING SEPARTE)
gsort persnr ausbild betnr begepi -level2
bysort persnr ausbild betnr begepi: gen nrA = _n if !missing(betnr)

gen jobdauer = endepi - begepi +1 if !missing(betnr)
gen jobdauer_kum = jobdauer
gen jobdauer_dup = 0

* AUXILIARY VARIABLE FOR DURATION OF PARALLEL EPISODES IN SAME ESTABLISHMENT (VOCATIONAL TRAINING SEPARATE)
replace jobdauer_dup = jobdauer if nrA != 1

* RUNNING TOTAL OF JOB DURATION (MINUS DURATION OF PARALLEL EPISODES)
replace jobdauer_kum = jobdauer_kum[_n-1] + jobdauer - jobdauer_dup if job ==1
gen tage_job = jobdauer_kum

drop jobdauer jobdauer_kum job jobdauer_dup nr nrA ausbild

*--------------------------------------*
* NUMBER OF BENEFIT RECEIPTS (anz_lst) *
*--------------------------------------*

* MARK EPISODE OF BENEFIT RECEIPT (DATA SOURCES LeH, LHG, XLHG)
gen quelleL = inlist(quelle, 2, 16, 19)

* ENUMERATE BENEFIT RECEIPTS WITHIN EPISODE
bysort persnr begepi endepi quelleL (quelle spell): gen nrL  = _n if quelleL
									
sort persnr spell

* COPY END DATE OF LAST BENEFIT RECEIPT TO SUBSEQUENT OBSERVATIONS
gen ende_vor = .
replace ende_vor = endepi[_n-1] if quelleL[_n-1] & persnr == persnr[_n-1]
replace ende_vor = ende_vor[_n-1] if missing(ende_vor) & persnr == persnr[_n-1]
format ende_vor %td

* MARK OBSERVATIONS THAT COUNT AS SEPARATE BENEFIT RECEIPTS
*  Only 1 per episode, separate benefit receipt if gap > 10 days 
gen lst = quelleL & nrL == 1 & (begepi - ende_vor > 10)

* RUNNING TOTAL OF BENEFIT RECEIPTS
gsort persnr begepi -lst
gen anz_lst = lst
replace anz_lst = anz_lst[_n-1] + lst if persnr == persnr[_n-1]
drop ende_vor lst

*------------------------------------------------*
* NUMBER OF DAYS WITH BENEFIT RECEIPT (tage_lst) *
*------------------------------------------------*

* DURATION OF BENEFIT RECEIPT (WITHOUT DURATION OF PARALLEL OBSERVATIONS)
gen lstdauer = 0
replace lstdauer = endepi - begepi +1 if quelleL ==1 & nrL ==1

* RUNNING TOTAL OF DAYS IN BENEFIT RECEIPT
bysort persnr (spell): gen tage_lst = sum(lstdauer) if nrL ==1	| quelleL ==0

* FILL VARIABLE FOR PARALLEL OBSERVATIONS
bysort persnr begepi endepi (nrL): replace tage_lst = tage_lst[1]

drop lstdauer quelleL nrL 

*-----------------------*
* ADJUST MISSING VALUES *
*-----------------------*

foreach var of varlist  ein_job tage_job {
	replace `var' = .n if quelle != 1
	replace `var' = .z if quelle ==1 & missing(`var')
}
replace ein_erw = .z if missing(ein_erw)


*--------------------------------------------------*
* LABEL THE GENERATED VARIABLES                    *
*--------------------------------------------------*

label language en
label variable ein_erw    "First day in employment"
label variable tage_erw   "Number of days in employment"
label variable ein_bet    "First day in establishment"
label variable tage_bet   "Number of days in establishment"
label variable ein_job    "First day in job"
label variable tage_job   "Number of days in job"
label variable anz_lst    "Number of benefit receipts"
label variable tage_lst   "Number of days with benefit receipt"

/*
label language de
label variable ein_erw    "Eintrittsdatum Erwerbsleben"
label variable tage_erw   "Dauer Erwerbsleben"
label variable ein_bet    "Eintrittsdatum Betrieb"
label variable tage_bet   "Dauer Betrieb"
label variable ein_job    "Eintrittsdatum Job"
label variable tage_job   "Dauer Job"
label variable anz_lst    "Anzahl bisheriger Leistungsbezüge"
label variable tage_lst   "Dauer bisheriger Leistungsbezüge"
*/

foreach var of varlist ein_erw tage_erw  ein_job tage_job  { 

	label language en
	lab def `var'_en .n ".n  n/a" .z ".z  no entry", add modify
	lab val `var' `var'_en
	numlabel `var'_en, add mask("#  ")

	label language de
	lab def `var'_de .n ".n  fehlt syst." .z ".z  k. Angabe", add modify
	lab val `var' `var'_de
	numlabel `var'_de, add mask("#  ")

}



compress
save "$data/bio.dta", replace
