
****************************************************************************************************
**** Create annual dataset for job loss regressions: job lost yes or no, if so when, date of birth, firm, the average wage, and average earnings
****************************************************************************************************
clear
*set max_memory 50g
cap log close

log using "$log/4_2_data_annual_regressions.log", replace
use "$data/monthly.dta", clear

tsset persnr caldate, monthly

set seed 123456789

// drop half the dataset due to memory constraints
bys persnr: gen sample=runiform() if _n==1
bys persnr: egen sample2=max(sample)
count
keep if sample2<.5
count
drop sample sample2


*merge with mass layoff indicators
merge n:1 calyr betnr using "$data/masslayoff.dta"
drop if _merge==2
drop _merge
tsset persnr caldate, monthly

* make it a balanced panel (the data has gaps which we want to assign zero earnings and employment to)
compress
count
tsfill
count
replace emp=0 if (emp==. |emp==.n)

* earnings
replace ave=0 if (ave==. | ave==.n) // zero earnings if no earnings
replace ave_b=0 if (ave_b==. | ave==.n)

* separation risk over the year
bys persnr calyr: egen emp_sum=total(emp) // number of months employed in a year
bys persnr calyr: egen jobloss_sum=total(jobloss) // number of jobloss events
bys persnr calyr: egen jobloss_sum_ue=total(jobloss_ue) // number of jobloss to unemployment events

tsset persnr caldate, monthly

gen sep_risk = .
replace sep_risk=jobloss_sum/emp_sum if emp_sum >0 // separation risk
gen sep_risk_ue = .
replace sep_risk_ue =jobloss_sum_ue/emp_sum if emp_sum >0 // separation risk into ue

drop jobloss_sum*

* create an indicator that takes zero if no future work within three years after jobloss
bys persnr: gen xperienc=sum(emp) //cumulative months an indvidual has been employed
bys persnr: gen reemp_check=F37.xperienc-xperienc if jobloss==1 // this variable takes zero if no added experience three years after jobloss
gsort persnr -caldate
bys persnr: gen fxperienc=sum(emp) //cumulative months will still be employed
bys persnr: replace reemp_check=0 if fxperienc==1 & jobloss==1 // now also takes zero if no future employment

sort persnr caldate
tsset persnr caldate

* now we set jobloss to 0 when workers dont return to work (within three years)
replace jobloss=0 if reemp_check==0
replace jobloss_ue=0 if reemp_check==0
replace jobloss_ten2=0 if reemp_check==0
replace jobloss_ten_ue2=0 if reemp_check==0

* occupation and industry in next job after jobloss
gen nextocc=beruf if emp==1
gen nextind=w93_3 if emp==1
gen nextemp=betnr if emp==1
gsort persnr -caldate
bys persnr: replace nextocc=nextocc[_n-1] if emp==0
replace nextocc=nextocc[_n-1] if jobloss==1
bys persnr: replace nextind=nextind[_n-1] if emp==0
replace nextind=nextind[_n-1] if jobloss==1
bys persnr: replace nextemp=nextemp[_n-1] if emp==0
replace nextemp=nextemp[_n-1] if jobloss==1

replace nextocc=. if jobloss~=1
replace nextind=. if jobloss~=1
replace nextind=. if jobloss~=1
gen sameocc=1 if nextocc==beruf & beruf~=.
gen sameind=1 if nextind==w93_3 & w93_3~=.
gen sameemp=1 if betnr==nextemp & betnr~=.

tab sameind
tab sameemp
count if jobloss==1
count if jobloss_ue==1
count if jobloss_ue==1 & sameind==1 & sameocc==1
drop next*

gen jobloss_noswitch=jobloss_ue if sameind==1 & sameocc==1

tsset persnr caldate
sort persnr caldate

* indicators for job lossf (1 if at least one job loss), along with earnings and wages, job information pertaining to january spell
collapse  (max) jobloss* av* sep* (first) gebjahr frau bild betnr beruf jten masslo, by(persnr calyr)

sort persnr calyr
tsset persnr calyr

gen age=calyr-gebjahr

* generate a variable which indicates whether person lost job in a given year
rename jobloss_ten_ue jobloss_4
rename jobloss_ten jobloss_3
rename jobloss_ue jobloss_2
rename jobloss jobloss_1

* compute alternative jobloss dummy that applies the masslayoff criterion
sort persnr calyr
tsset persnr calyr

gen jobloss_5=(jobloss_3==1 & masslo==1) // dont impose ui but mass layoff
bys persnr: gen jobloss_6=((jobloss_3==1 | F1.jobloss_3==1 | F2.jobloss_3==1) & masslo==1) // approximate the davis von wachter classification

* industry-occupation stayers
bys persnr: gen jobloss_7=jobloss_noswitch // impose no industry or occupation switch
drop jobloss_no*

gen jobloss_8=(jobloss_1==1 & masslo==1) // mass layoff
gen jobloss_9=(jobloss_2==1 & masslo==1) // mass layoff & ui


label var jobloss_1 "jobloss"
label var jobloss_2 "jobloss into ui"
label var jobloss_3	"jobloss w/ high tenure"
label var jobloss_4 "jobloss w/ high tenure into ui"
label var jobloss_5 "jobloss w/ mass layoff w high tenure"
label var jobloss_6 "jobloss w/ mass layoff w high tenure and (Davis, von Wachter) classification"
label var jobloss_7 "like 4 but return into same industry and occupation (but not same employer)"
label var jobloss_8 "jobloss and mass layoff"
label var jobloss_9 "jobloss into ui w/ mass layoff"


sum jobloss*

tsset persnr calyr , yearly
compress
count
save "$data/annual.dta", replace




