// This runs displacement regressions. Instead of showing all the regression tables I print the relevant coefficients into a matrix along with the number of observations used
// The first part of the file constructs, for each case, a dataset regression_sample_full which  does the displacement-year stacking described in the paper
// It does so for 6 different models (baseline (1), plus 5 different robustness exercisees) but the structure is always the same. It also prints a few summary stats
// To run on simulated data change simul=1 below

// note: the file produces additional versions of the controls that can alternatively be used. It produces all the results that are reported in the paper but these additional controls can be used to generate additional results.

clear

cap log close
set more off
set seed 123456789

*** Set the number of leads and lags - 6 leads (the last one being the layoff year) 20 lags
global ndumm 26
global leadslags 1/${ndumm}
*** Set First and Last Displacement calyr - Full Thing: 78-10 (such that 3 year pre separation tenure restriction can be satisfied)
global beg 1980
global end 2010
global ny = ${end}-${beg}+1

local simul=1 // set this to 1 to run the code on simulated model based data
local mod=3 // set this to 2 or three for the simulated data from the no heterogeneity models (keep to 1 when running on actual data)

*****************
* several cases: 1) baseline, 2) Mass Layoff Separators, 3) Deal w/ Part-Time Earnings Differently, 4) Davis von Wachter specification, 5) men only, 6) only industry-occupation stayers
*****************
if `simul'==0{
log using "$log/6_regression_520.log", replace
} 
if `simul'==1{
log using "$log/6_regression_520sim`mod'.log", replace
} 

//quietly{
forval cas=1/1{

use "$data/annual.dta", clear 

bys persnr: gen sample=runiform() if _n==1
bys persnr: egen sample2=max(sample)
count
keep if sample2<1
count
drop sample sample2

if `simul'==1{
if `mod'==1{
use "$simul/annual_simul.dta", clear // for simulated data
*use "$simul/annual_simul_small.dta", clear 
*use "$simul/annual_simul_vsmall.dta", clear 

}
if `mod'==2{
use "$simul/annual_simul_het_delta.dta", clear // for simulated data
}
if `mod'==3{
use "$simul/annual_simul_het_s.dta", clear // for simulated data
}
}

// for the stacking: count individuals as 1,2,3,...
egen hoy=tag(persnr)
gen hoy3=sum(hoy)
sum hoy3
local wt=r(max) // number of distinct individuals
replace persnr=hoy3

gen persorig=persnr
gen yearorig=calyr

tsset persnr calyr, yearly

// for all years without earnings we assign zero earnings
replace ave=0 if (ave==. | ave==.n)

// whenever there are at least three consecutive years with zero earnings drop the corresponding observations
*drop if ((ave==0 & L.ave==0) | (ave==0 &F.ave==0))
drop if ((ave==0 & L.ave==0 & L2.ave==0) | (ave==0 & L.ave==0 & F.ave==0)| (ave==0 & F.ave==0 & F2.ave==0))

tsset persnr calyr, yearly

quietly{
if `cas'==5{
sum frau
keep if frau==0
}
*******************************************************************
************ this block applies sample selection criteria: for each displacement year, create a separate dataset
******************************************************************
forval y=$beg/$end{
preserve

local cc=`y'-$beg
replace persnr=persnr+`wt'*`cc' // in the next displacement year dataset assign different id

keep if age>17 & age<66

*** Create A Dummy that Indicates Jobloss in Displacement Year -- or 2 years after like in DvW -- 
gen jobloss_ind=jobloss_4 // jobloss4 is high tenure into ui

*** alternatively, mass layyoff indicator
if `cas'==2{
replace jobloss_ind=jobloss_9
}
*** alternatively, only job losers who turn out to be occupation-industry stayers 
if `cas'==6{
replace jobloss_ind=jobloss_7
}
*** keep only jobloss events in displacement year
if `cas'!=4{
replace jobloss_ind=0 if calyr!=`y' 
}
*** like in DvW, keep job losers also in two years after
if `cas'==4{
replace jobloss_ind=0 if calyr!=`y' & calyr!=`y'+1 & calyr!=`y'+2
}
*** group in treatment vs control group
bys persnr: egen lossiny=max(jobloss_ind)

*** workers must have been full time employed, ie wage equal earnings (with wage above some min threshold)
if `simul'==0{
gen fltm=1 if ave==avw & avw>24
}
if `simul'==1{
gen fltm=1 if ave==avw & avw>.1
}
bys persnr: egen mine = sum(fltm) if inrange(calyr, `y'-3,`y'-1)
drop if mine<3

*** must also have some positive earnings in displacement year
bys persnr: gen pose=1 if ave>0 & calyr==`y'
bys persnr: egen poseind=max(pose)
keep if poseind==1

*** Keep only those who've been for three years at the same firm
bys persnr: gen firmten2=jten if calyr==`y'
bys persnr: egen firmten3=max(firmten2) 
drop if firmten3<36
drop firmt*

*** keep only prime-age worker at displacement
gen keepage=age if calyr==`y'
bys persnr: egen keepage2=max(keepage)
keep if keepage2<55 & keepage2>25
drop keepage*

*** take logs and introduce alternative pay measure
gen lavw=log(avw)
gen lave=log(ave)

* use alternative pay measure (which includes some part-time pay)
if `cas'==3{
replace avw=avw_b
replace ave=ave_b
replace lavw=log(avw_b)
replace lave=log(ave_b)
}

*** Compute the mean wages for the past three years before displacement year and for all years prior to displacement

** wages, logs
bys persnr: egen wbar = mean(lavw) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen wbar2= max(wbar)

bys persnr: egen wbar3 = mean(lavw) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen wbar4= max(wbar3)

** wages, levels
bys persnr: egen wbar5 = mean(avw) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen wbar6= max(wbar5)

bys persnr: egen wbar7 = mean(avw) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen wbar8= max(wbar7)

** earnings, logs
bys persnr: egen ebar = mean(lave) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen ebar2= max(ebar)

bys persnr: egen ebar3 = mean(lave) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen ebar4 = max(ebar3)

** earnings, levels
bys persnr: egen ebar5 = mean(ave) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen ebar6= max(ebar5)

bys persnr: egen ebar7 = mean(ave) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen ebar8 = max(ebar7)

** job loss, two alternative measures -- note that recent job loss is zero by sample construction
bys persnr: egen jl1bar = mean(jobloss_1) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen jl1bar2= max(jl1bar)

bys persnr: egen jl1bar3 = mean(jobloss_1) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen jl1bar4= max(jl1bar3)

bys persnr: egen jl2bar = mean(jobloss_2) if inrange(calyr, `y'-3,`y'-1) 
bys persnr: egen jl2bar2= max(jl2bar)

bys persnr: egen jl2bar3 = mean(jobloss_2) if inrange(calyr, `y'-50,`y'-1) 
bys persnr: egen jl2bar4= max(jl2bar3)

drop wbar wbar3 wbar5 wbar7 ebar ebar3 ebar5 ebar7 jl1bar jl2bar jl1bar3 jl2bar3

*** Create Displacement Dummies
gen sincedis=calyr-`y' // years since displacement year

tsset persnr calyr, yearly

* assign the displacement dummies for those with job loss in displacement year
foreach k of numlist $leadslags {
	gen Dl`k'=0
	replace Dl`k'=1 if lossiny==1 & sincedis==`k'-6
}

*in the next displacement year dataset assign different year so that its truly different stacked datasets
local addy=$ny*`cc'
replace calyr=calyr+`addy'

drop mine*  jten beruf sincedis
compress
save "$data/regression_sample_`y'", replace
*noi display "`y'"
restore
}
}

use "$data/regression_sample_$beg", clear
local beg2=$beg+1
forval y=`beg2'/$end{
append using "$data/regression_sample_`y'"
}

// gen percentiles of the controls (rather than having linear controls)
xtile frstw=wbar2, nq(10)
xtile lstw=wbar4, nq(10)
xtile frste=ebar2, nq(10)
xtile lste=ebar4, nq(10)

xtile frstw_lvl=wbar6, nq(10)
xtile lstw_lvl=wbar8, nq(10)
xtile frste_lvl=ebar6, nq(10)
xtile lste_lvl=ebar8, nq(10)



tsset persnr calyr
compress
save "$data/regression_sample_full", replace


use "$data/regression_sample_full", clear

********************************************************************
***** some summary stats for the baseline
*********************************************************************

if `cas'==1 & `simul'==0{
noisily{
display "Some Summary Stats"

keep if yearorig>$beg & yearorig<=$end // sample years

tab yearorig
sum Dl6 // number of job loss events
tab Dl6 yearorig // number of job loss events by calyr
bys Dl6: sum age // age distribution at job loss
tab Dl6 frau  // sex distribution job loosers
tab bild Dl6 // education group
bys Dl7: sum avw // average wage at job loss
bys Dl6: sum avw // average wage at job loss
bys Dl5: sum avw // prior
bys Dl4: sum avw
tsset persnr calyr
}
}

****************************************************************************************************
********************* REGRESSIONS
****************************************************************************************************

*******************************
*** EARNINGS
*******************************

*** baseline
reghdfe lave Dl*, a(frste lste age calyr) vce(cluster persorig yearorig)
*cgmreg lave Dl* i.firste i.lste i.age i.calyr, cluster(persorig yearorig)
mat coef=e(b)
mat c_le_base=coef[1,1..26]
mat var=e(V)
mat var2=vecdiag(var)
mat c_le_base_v=var2[1,1..26] 
mat c_le_base_n=e(N)

*** Levels (so we can include zeros for earnings)
reghdfe ave Dl*, a(frste_lvl lste_lvl age calyr) 

mat c_e_base_n=e(N)
mat coef=e(b)
mat coefD=coef[1,1..26]

mat var=e(V)
mat var2=vecdiag(var)
mat coef_v=var2[1,1..26] 

* the next block uses the coefficients on the layoff dummies to construct counterfactual earnings for the treatment group (no treatment)
* along the way it computes the present value of realized and counterfactul earnings to compute the PV earnings loss

mat loss=J(1,26,0)
mat loss2=J(28,26,0)
mat loss_v=J(1,26,0)

quietly{
gen bet=.
gen aved2=0
gen ealtd2=0
forval i=1/26{
gen ealt=.

*gen loss=.
sum ave if Dl`i'==1
mat meane=r(mean)
qui semean ave if Dl`i'==1
mat meanv=r(semean)
mat loss[1,`i']=-coefD[1,`i']/(meane[1,1]-coefD[1,`i'])
mat loss_v[1,`i']=loss[1,`i']^2*((meane[1,1]-coefD[1,`i'])^(-2)*coef_v[1,`i']+coefD[1,`i']^(-2)*(coef_v[1,`i']+meanv[1,1]))

replace ealt=ave-Dl`i'*coefD[1,`i'] if Dl`i'==1
*replace loss=ave/ealt
*sum loss, det
*mat loss[1,`i']=r(mean)
* present value earnings over next 20 years both realized and counterfactual
local jj=`i'-6
replace bet=.95^`jj' if Dl`i'==1 & `i'>5	
gen aved=ave*bet
gen ealtd=ealt*bet
if `i'>5{
replace aved2=F`jj'.aved+aved2 if Dl6==1
replace ealtd2=F`jj'.ealtd+ealtd2 if Dl6==1
}
*drop ealt loss aved ealtd
drop ealt aved ealtd
}
}
gen pvloss=aved2/ealtd2 

noi sum pvloss if Dl6==1
mat c_e_base=loss
mat c_e_base_v=loss_v
*noi mat list loss

*******************************
*** WAGES
*******************************

* baseline
reghdfe lavw Dl*, a(frstw lstw age calyr) vce(cluster persorig yearorig)
*cgmreg lavw Dl* i.firstw i.lstw i.age i.calyr, cluster(persorig yearorig)
mat coef=e(b)
mat c_lw_base=coef[1,1..26]
mat var=e(V)
mat var2=vecdiag(var)
mat c_lw_base_v=var2[1,1..26] 
mat c_lw_base_n=e(N)


*** Levels (so we can include zeros for earnings)
reghdfe avw Dl*, a(frstw_lvl lstw_lvl age calyr)

mat c_w_base_n=e(N)
mat coef=e(b)
mat coefD=coef[1,1..26]

mat var=e(V)
mat var2=vecdiag(var)
mat coef_v=var2[1,1..26] 

* the next block uses the coefficients on the layoff dummies to construct a counterfactual wage for the treatment group (no treatment)

mat loss=J(1,26,0)
mat loss2=J(28,26,0)
mat loss_v=J(1,26,0)

quietly{
replace bet=.
gen avwd2=0
gen waltd2=0
forval i=1/26{
gen walt=.
*gen loss=.

sum avw if Dl`i'==1
mat meanw=r(mean)
qui semean avw if Dl`i'==1
mat meanv=r(semean)
mat loss[1,`i']=-coefD[1,`i']/(meanw[1,1]-coefD[1,`i'])
mat loss_v[1,`i']=loss[1,`i']^2*((meanw[1,1]-coefD[1,`i'])^(-2)*coef_v[1,`i']+coefD[1,`i']^(-2)*(coef_v[1,`i']+meanv[1,1]))

replace walt=avw-Dl`i'*coefD[1,`i'] if Dl`i'==1
* present value wages over next 20 years both realized and counterfactual
local jj=`i'-6
replace bet=.95^`jj' if Dl`i'==1 & `i'>5	
gen avwd=avw*bet
gen waltd=walt*bet
if `i'>5{
replace avwd2=F`jj'.avwd+avwd2 if Dl6==1
replace waltd2=F`jj'.waltd+waltd2 if Dl6==1
}
drop walt avwd waltd
}
}
mat c_w_base=loss
mat c_w_base_v=loss_v

replace pvloss=avwd2/waltd2 

noi sum pvloss if Dl6==1
*noi mat list loss

*******************************
*** JOB LOSS
*******************************

gen jl_2=jobloss_2 
gen jl_1=jobloss_1 

*baseline
reghdfe jl_2 Dl* jl2bar2 jl2bar4, a(age calyr) vce(cluster persorig yearorig)
*cgmreg jl_2 Dl* i.firste i.lste i.age i.calyr, cluster(persorig yearorig)
mat coef=e(b)
mat c_jl2_base=coef[1,1..26]
mat var=e(V)
mat var2=vecdiag(var)
mat c_jl2_base_v=var2[1,1..26]
mat c_jl2_base_n=e(N)

* alternative job loss indicator
reghdfe jl_1 Dl* jl1bar2 jl1bar4, a(age calyr) vce(cluster persorig yearorig)
mat define coef=e(b)
mat c_jl1_base=coef[1,1..26]
mat c_jl1_n=coef[1,1..26]



***************************************
********** list all results
***************************************

noi display "Results for Case `cas':"
noi display "Response of Log Earnings (LE), Earnings (E), Log Wages (LW), Wages (W), Job Loss (JL2). Ending with _v stores variance of estimates"
mat results`cas'=[ c_le_base \ c_le_base_v \ c_e_base \ c_e_base_v \ c_lw_base \ c_lw_base_v \ c_w_base \ c_w_base_v \ c_jl2_base \c_jl2_base_v ]'
mat colnames results`cas'= LE LE_V E E_V LW LW_V W W_V JL JL_V
mat r`cas'=results`cas''
noi mat list r`cas', format(%8.5f)

mat n`cas'=[ c_le_base_n[1,1] \ c_e_base_n[1,1] \ c_lw_base_n[1,1] \c_w_base_n[1,1] \ c_jl2_base_n[1,1] ]'
mat colnames n`cas'= N_LE_Base N_E_Base N_LW_Base N_W_Base JL_Base
noi display "This shows the number of observations used in the regresssions behind the previous matrix"
noi mat list n`cas'

if `simul'==1 & `mod'==1{
mat2txt, matrix(results`cas') saving(log/losses`mod') replace
}
if `simul'==1 & `mod'>1{
mat2txt, matrix(results`cas') saving(log/losses`mod') replace
}

}
*}
