*File of the Project "670" - 32_moments.do
*This computes various data moments on a monthly panel 
*I execute the computation quietly and print all (13) moments at the very end in a matrix that also reports the number of observations used in the computation of each moment

/*
List of Moments
1. Job Finding Rate of the Unemployed
2. Monthly of Job-to-Job 
3. Mean Separation Rate (rate of job loss into unemployment)
4.-6. Duration Dependence in the Separation Rate (measuring the effect of job tenure on separation risk)
7. The effect of the length of the preceeding unemployment spell on the next wage
8. Relating Layoff Risk at the firm level and wages
9. Mean wage of individuals that just came out of unemplyment relative to everyone
10./11 Variance and Skewness of Residualized Log Wages
12. Average Wage Growth
13. Relating Wage Jump upon job-to-job move to duration of preceeding employment spell
*/

clear

cap log close
log using "$log/32_moments.log", replace

set more off

use "$data/monthly.dta", clear
format * %8.4f

set seed 123456789

bys persnr: gen sample=runiform() if _n==1
bys persnr: egen sample2=max(sample)
count
keep if sample2<.5
count
drop sample sample2


sort persnr caldate
*******************************************************************************
********************* Compute the Moments
*******************************************************************************
quietly{
** define the time horizon as a year
local hor=12

** * 1st and 3rd MOMENT: job finding and losing rates and implied urate

*job finding rate: found job if coming from UI system (withitn last three months)
bys persnr: gen jobfound_ue=(emp[_n+1]==1 & emp[_n]==0 & erwstat[_n]<3)
sum jobfound_ue if emp[_n]==0 & erwstat[_n]<3
mat jfr_ue=r(mean)
mat jfr_ue_n=r(N)
mat jfr_ue_v=r(Var)/r(N)


* 3rd Moment Job loss rate: lost job if moving into UI or short subsequent spell
sum jobloss_ue if emp==1, det
mat jlr_ue=r(mean)
mat jlr_ue_n=r(N)
mat jlr_ue_v=r(Var)/r(N)

*urate implied by flow balance:
mat uerate=jlr_ue[1,1]/(jlr_ue[1,1]+jfr_ue[1,1])

* 2nd MOMENT: Mean EE Rate
sum jobtojob if emp==1
mat eerate= r(mean)
mat eerate_n=r(N)
mat eerate_v=r(Var)/r(N)


* 4th and 5th and 6th MOMENT: Duration Dependence

* create controls
recode age (16/24=1) (25/34=2) (35/44=3) (45/54=4) (55/64=5) (nonmiss=6), gen(age2)
gen frau_bild=frau*bild

gen eten_q=ceil(eten/3) //quarterly
preserve
drop if (eten_q>40 | eten_q==.  | eten==0)
xi: reg jobloss_ue i.eten_q i.calmn i.calyr i.frau i.bild i.frau_bild i.age2  // regress on duration dummy
mat beta=e(b)
mat bvar=e(V)
mat list beta
mat bet1=(beta[1,1]+beta[1,2]+beta[1,3])/4 //first quarter ommited category

forval j=2/10{
mat bet`j'=(beta[1,`j'*4-4]+beta[1,`j'*4-3]+beta[1,`j'*4-2]+beta[1,`j'*4-1])/4-bet1
mat bet_v`j'=(bvar[`j'*4-4,`j'*4-4]+bvar[`j'*4-3,`j'*4-3]+bvar[`j'*4-2,`j'*4-2]+bvar[`j'*4-1,`j'*4-1])/16
}
mat DD=[ bet2 \ bet5 \ bet10]
mat DD_v=[ bet_v2 \ bet_v5 \ bet_v10]
mat DD_n=[e(N) \ e(N) \e(N)]

mat DD_full=bet1
forval j=2/10{
mat DD_full=[ DD_full \ bet`j']
}
restore

* 7th MOMENT: log re-employment wage regressed on previous unempl duration
gen ind=(emp[_n]==1 & emp[_n-1]==0 & persnr[_n-1]==persnr[_n] & udur_ue!=. & udur_ue<24) //newly employed, know duration
reg logwage meanwage udur_ue if ind==1

reghdfe logwage meanwage udur_ue if ind==1,a(age calyr bild frau)
mat wage_dur=e(b)

replace ind=(emp[_n]==1 & emp[_n-1]==0 & persnr[_n-1]==persnr[_n] & udur!=. & udur<24) //newly employed, know duration
reg logwage meanwage udur if ind==1

reghdfe logwage meanwage udur if ind==1,a(age calyr bild frau)
mat wage_dur=e(b)
mat wage_dur_v=e(V)
mat w_du_0c=wage_dur[1,2]
mat w_du_0c_v=wage_dur_v[2,2]
mat w_du_0c_n=e(N)

* 8th MOMENT: Layoff risk on wage

* regress firm level separation rate on worker wage and controls (eliminate outliers in terms of separation frequency both for firms and workers)
bys persnr: egen baru=mean(jobloss_ue) 
sum baru, det
sum firm_sep_ue, det
sum logwage, det

noi reg firm_sep_ue logwage meanwage baru if firm_sep_ue>0 & firm_sep_ue<=1 & baru<.08 

mat lw_sep=e(b)
mat lw_sep_v=e(V)
mat lw_firmsep=lw_sep[1,1]
mat lw_firmsep_v=lw_sep_v[1,1]
mat lw_firmsep_n=e(N)

*9th moment
* w0/w

*raw
sum avw if eten==1 & emp==1 & avw>0
mat mean_wage_hired=r(mean)
sum avw if avw>0 & emp==1
mat mean_wage=r(mean)
mat w0_w = mean_wage_hired[1,1]/mean_wage[1,1]
mat w0_w_n=r(N)

*residualized
//bys persnr: gen jobfound_ue=(emp[_n+1]==1 & emp[_n]==0 & (erwstat[_n]<3 | erwstat[_n-1]<3 | (erwstat[_n-2]<3 & emp[_n-1]==0)))

reghdfe avw  if eten==25 & emp==1 & avw>0, a(frau bild age caldate) res(resi_tent)
gen avw_rep_hired=avw-resi_tent
drop resi_tent

reghdfe avw if emp==1 & avw>0, a(frau bild age calyr) res(resi_tent)
gen avw_rep=avw-resi_tent
drop resi_tent

sum avw_rep_hired  if eten==25 & emp==1 & avw>0, det
mat w0_w_res_n=r(N)
mat mean_wage_hired=r(mean)
mat mean_wage_hired_v=r(Var)/r(N)

sum avw_rep if avw>0 & emp==1, det
mat mean_wage=r(mean)
mat mean_wage_v=r(Var)/r(N)
mat w0_w_res = 1-mean_wage_hired[1,1]/mean_wage[1,1]


gen avw_rep_inv=1/avw_rep if avw>0 & emp==1
sum avw_rep_inv
mat mean_wage_inv=r(mean)
mat mean_wage_inv_v=r(Var)/r(N)

mat w0_w_res_v=mean_wage_hired_v*mean_wage_inv_v+mean_wage_hired_v*(mean_wage_inv*mean_wage_inv)+mean_wage_inv_v*(mean_wage_hired*mean_wage_hired) // just computing the variance of a product of random variables
mat list w0_w_res_v
mat list w0_w_res

* 10th & 11th moment: 90-50 and 50-10 of residualized log wages
gen lw=logwage-meanwage
sum lw, detail
sum lw if lw>r(p1)& lw<r(p99), det
mat fiftyten=r(p50)-r(p10)
mat ninetyfifty=r(p90)-r(p50)
mat fiftyten_n=r(N)
mat ninetyfifty_n=r(N)

*alternative
reghdfe logwage, a(persnr calyr) res(resi)
sum resi, det
sum resi if resi>r(p1)& resi<r(p99), det
mat fiftyten_2=r(p50)-r(p10)
mat ninetyfifty_2=r(p90)-r(p50)
mat fiftyten_2_n=r(N)
mat ninetyfifty_2_n=r(N)

// bootstrap empirical standard error or percentiles
sqreg resi if resi>r(p1)& resi<r(p99), quantile(10 50 90)
mat perc_v=e(V)
mat fiftyten_2_v=perc_v[1,1]+perc_v[2,2]
mat ninetyfifty_2_v=perc_v[2,2]+perc_v[3,3]

_pctile resi, p(1)
mat wagedis=r(r1)'
forval ii=2/99{
_pctile resi, p(`ii')
mat wagedis=[wagedis' \ r(r1)]'
}

* 12th moment
* wage growth while employed (relative to aggregate wage growth)
merge n:1 calyr using "$data/wg_agg.dta"
drop _merge
sort persnr caldate
tsset persnr caldate

local hor=12
gen wget=(avw/avw[_n-`hor'])/wg_agg if (emp[_n-`hor']==1 & emp[_n]==1 & eten[_n-`hor']==eten[_n]-`hor' & persnr[_n]==persnr[_n-`hor'] & eten[_n]-`hor' >0)
sum wget, det
sum wget if wget>r(p1) & wget<r(p99), det
mat w_growth_e = r(mean)
mat w_growth_e_n=r(N)
mat w_growth_e_v = r(Var)/r(N)

*13th moment regress starting wage on starting wage last job and time that's passed since
gen newwage = log(avw[_n+1]) if jobtojob==1
* old wage when worker was hired
gen starting_wage= avw[_n-jten] if (jobtojob==1 & avw[_n-jten]>0 & emp[_n-jten]==1)
gen l_starting_wage= log(starting_wage)
reg newwage jten l_starting_wage if jten<96
mat upcoeff=e(b)
mat upcoeff_v=e(V)
mat up=upcoeff[1,1]
mat up_v=upcoeff_v[1,1]
mat up_n=e(N)

}

***************************************************
* PRINTING OUT THE RESULTS
***************************************************

*The matrix results contains the 13 data moments that I compute. I list the number of observations right underneath 

mat results=[ jfr_ue \ eerate \ jlr_ue \  DD \ w_du_0c \lw_firmsep\ w0_w_res \  fiftyten_2 \ninetyfifty_2 \ w_growth_e \  up]
mat results_v=[ jfr_ue_v \ eerate_v \ jlr_ue_v \  DD_v \ w_du_0c_v \lw_firmsep_v\ w0_w_res_v \ fiftyten_2_v \ninetyfifty_2_v \ w_growth_e_v \  up_v]
mat results_n=[ jfr_ue_n \ eerate_n \ jlr_ue_n \  DD_n \ w_du_0c_n \lw_firmsep_n\ w0_w_res_n \ fiftyten_2_n \ninetyfifty_2_n \ w_growth_e_n \  up_n]
mat res = [results' \ results_v' \ results_n']
mat colnames res=  UErate EErate EUrate DD1 DD2 DD3 W0_Dur W_on_Firmsep w0_w_res 50-10v2 90-50v2 wg_e wg_jj_time
mat rownames res= Moments VarianceOfMoments Observations_Used
mat list res, format(%11.10f)

* in different format:
mat list results_n 


*This computes two additional set of results, again with the number of observations used reportedly separately
display "Full list for Duration Dependence -- The scalar after prints the # of obs used in the regression"
mat DD_full=DD_full'
mat list DD_full , format(%8.5f)
mat list DD_n

display "Full list for Wage Distribution -- The scalar after prints the # of obs used in the computation"
mat list wagedis, format(%3.2f)
mat list fiftyten_n

log close
